/*
gnaughty - porn movie downloader
Copyright (C) 2002 David Moore

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/
#include <gtk/gtk.h>
#include <string.h>
#include "gui.h"
#include "parselinks.h"
#include "icons.h"
#include "callbacks.h"

extern GnaughtyPrefs g_prefs;
extern const gchar *porntypes[];
extern const gchar* statusdescriptions[];
extern GladeXML *GnaughtyXML;

static GHashTable* gs_TreeRowReferences;

GladeXML* gnaughty_gui_new_mainwin(gnaughty_config key)
{
	GnaughtyLinksListWidget* links_list;
	GnaughtyDownloadsListWidget* downloads_list;
	GList* list;

	gs_TreeRowReferences = g_hash_table_new(NULL, NULL);

	/* give all windows the same icon (in theory) */
	list = g_list_prepend (NULL, gdk_pixbuf_new_from_inline (-1, gnaughty64_icon, FALSE, NULL));
	list = g_list_prepend (list, gdk_pixbuf_new_from_inline (-1, gnaughty48_icon, FALSE, NULL));
	list = g_list_prepend (list, gdk_pixbuf_new_from_inline (-1, gnaughty32_icon, FALSE, NULL));
	list = g_list_prepend (list, gdk_pixbuf_new_from_inline (-1, gnaughty16_icon, FALSE, NULL));
	gtk_window_set_default_icon_list (list);
	g_list_free (list);

	GnaughtyXML = glade_xml_new(GNAUGHTY_GLADEDIR "gnaughty.glade", NULL, NULL);
	glade_xml_signal_autoconnect(GnaughtyXML);

	// Set some options on main window
	GtkWindow* mainwin = GTK_WINDOW(glade_xml_get_widget(GnaughtyXML, "gnaughty_mainwindow"));
	// To set the icon of the download button, we have to find that button first
	GtkToolButton* download_button = GTK_TOOL_BUTTON(glade_xml_get_widget(GnaughtyXML, "download_button"));
	GTK_IS_TOOL_BUTTON(download_button);
	//GList* button_children = gtk_container_get_children(GTK_CONTAINER(download_button));
	//GtkHButtonBox* box = g_list_nth_data(button_children, 0);
	GtkWidget* image;
	image = gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_inline( 4096, download_icon, FALSE, NULL));
	gtk_tool_button_set_icon_widget (download_button, image);
	//gtk_box_pack_start(GTK_BOX(box), image , TRUE, TRUE, 0);
	gtk_widget_show(image);

	// Create 'links' list
	links_list = create_links_list_widget();
	gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(GnaughtyXML, "links_scrolled_window")), links_list->list);

	// Create 'downloads' list
	downloads_list = create_downloads_list_widget();
	gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(GnaughtyXML, "downloads_scrolled_window")), downloads_list->list);

	// Fill option menu
	int i;
	GtkWidget* menu = gtk_menu_new();
	GtkWidget* menuitem;
	for(i = 0 ; i < NUMPORNTYPES ; i++)
	{
		menuitem = gtk_menu_item_new_with_label(porntypes[i]);
		gtk_menu_shell_append (GTK_MENU_SHELL(menu), menuitem);
		gtk_widget_show (menuitem);
	}
	gtk_option_menu_set_menu(GTK_OPTION_MENU(glade_xml_get_widget(GnaughtyXML, "types_optionmenu")), menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(GnaughtyXML, "types_optionmenu")), MOVIEENTRY);
	g_signal_connect(GTK_OPTION_MENU(glade_xml_get_widget(GnaughtyXML, "types_optionmenu")), "changed", G_CALLBACK(changeporntype), NULL);

	// Preferences window should hide when closed
	
	//g_signal_connect_swapped(G_OBJECT(glade_xml_get_widget(GnaughtyXML, "close_prefs_button")), "clicked", G_CALLBACK(gtk_widget_hide_all), glade_xml_get_widget(GnaughtyXML, "preferences_window"));
	g_signal_connect_swapped(G_OBJECT(glade_xml_get_widget(GnaughtyXML, "preferences_window")), "delete_event", G_CALLBACK(delete_event), glade_xml_get_widget(GnaughtyXML, "preferences_window"));

    return GnaughtyXML;
}

void gnaughty_gui_set_download_progress(download_t* download, gdouble status)
{
	//gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(glade_xml_get_widget(GnaughtyXML, "totalprogress")), status/100);
	//gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(GnaughtyXML, "download_window")), g_strdup_printf("Downloading - %.0f%%", status * 100));

	GtkWidget* scrolledwindow = glade_xml_get_widget(GnaughtyXML, "downloads_scrolled_window");
	GList* children = gtk_container_get_children(GTK_CONTAINER(scrolledwindow));
	GtkListStore* store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(children->data)));

	GtkTreeRowReference* reference = g_hash_table_lookup(gs_TreeRowReferences, download);
	assert(reference);

	status == 100 ? (download->status = STATUS_FINISHED) : (download->status = STATUS_DOWNLOADING);

	GtkTreePath* path = gtk_tree_row_reference_get_path(reference);
	GtkTreeIter treeiter;
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &treeiter, path);
	gtk_list_store_set(store, &treeiter,
		DOWNLOADS_STATUS_COLUMN, statusdescriptions[download->status],
		DOWNLOADS_PROGRESS_COLUMN, status / 100,
		-1
	);

	gtk_tree_path_free (path);
	return;
}

gdouble gnaughty_gui_get_download_progress()
{
	gdouble status=0;
	status = gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(glade_xml_get_widget(GnaughtyXML, "totalprogress")));
	return status;
}

void gnaughty_gui_set_download_status(gchar* status)
{
	GtkStatusbar* statusbar = GTK_STATUSBAR(glade_xml_get_widget(GnaughtyXML, "statusbar"));
	guint id = gtk_statusbar_get_context_id(statusbar, "");

	gtk_statusbar_pop(statusbar, id);
	gtk_statusbar_push(statusbar, id, status);
	g_free (status);
	return;
}

void processitem(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	GList **urls = data;
	gchar *url;
	gchar *desc;
	gchar *tmp;

	gtk_tree_model_get(model, iter, LINKS_URL_COLUMN, &url, -1);
	gtk_tree_model_get(model, iter, LINKS_DESC_COLUMN, &desc, -1);
	if (strstr (desc, " videos ")) {
		tmp = g_strconcat (url, "/MOV/", NULL);
	} else {
		//Pictures then..
		tmp = g_strconcat (url, "/PIC/", NULL);
	}
	*urls = g_list_append(*urls, tmp);
	g_free (desc);
	g_free (url);
}

/**
 * Returns a GList of *'s to gchar* 's
 */
GList* gnaughty_gui_get_selected_urls()
{
	GtkTreeSelection *selection;
	GList            *list = NULL;

	GtkWidget* scrolledwindow = glade_xml_get_widget(GnaughtyXML, "links_scrolled_window");
	GList* children = gtk_container_get_children(GTK_CONTAINER(scrolledwindow));
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(children->data));
	if (selection == NULL)
		return NULL;
	gtk_tree_selection_selected_foreach(selection, processitem, &list);

	return list;
}

/**
 * @param links: GList* of *'s to download_t's.
 */
void gnaughty_gui_add_links_to_download_list(GList* downloads)
{
	GList* iterator;
	gchar* new_url;
	GtkTreeIter treeiter;

	for(iterator = downloads ; iterator != NULL ; iterator = iterator->next) {
		download_t* current_download = (download_t*)(iterator->data);
		new_url = current_download->URL->str;
		GtkWidget* scrolledwindow = glade_xml_get_widget(GnaughtyXML, "downloads_scrolled_window");
		GList* children = gtk_container_get_children(GTK_CONTAINER(scrolledwindow));
		GtkListStore* store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(children->data)));
		assert(GTK_IS_LIST_STORE(store));
		gtk_list_store_append(store, &treeiter);
		gtk_list_store_set(store, &treeiter,
			DOWNLOADS_URL_COLUMN, new_url,
			DOWNLOADS_STATUS_COLUMN, statusdescriptions[current_download->status],
			DOWNLOADS_PROGRESS_COLUMN, 0.0,
			-1
		);
		// Store the treeiter with the download item
		GtkTreeRowReference* reference = NULL;
		GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &treeiter);
		reference = gtk_tree_row_reference_new(GTK_TREE_MODEL(store), path);
		g_hash_table_insert(gs_TreeRowReferences, current_download, reference);
		gtk_tree_path_free(path);
	}
}

void gnaughty_gui_add_movie_list_item(link_t* item)
{
	gchar *new_url;
	gchar *new_desc;
	GtkTreeIter iter;

	new_url=(item->url)->str;
	new_desc=(item->desc)->str;
	GtkWidget* scrolledwindow = glade_xml_get_widget(GnaughtyXML, "links_scrolled_window");
	GList* children = gtk_container_get_children(GTK_CONTAINER(scrolledwindow));
	GtkListStore* store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(children->data)));
	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, LINKS_DESC_COLUMN, new_desc, LINKS_URL_COLUMN, new_url, -1);
}

void gnaughty_gui_clear_movie_list()
{
	GtkWidget* scrolledwindow = glade_xml_get_widget(GnaughtyXML, "links_scrolled_window");
	GList* children = gtk_container_get_children(GTK_CONTAINER(scrolledwindow));
	GtkListStore* store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(children->data)));
	gtk_list_store_clear (store);
}

void gnaughty_gui_remove_download(download_t* download)
{
	GtkTreeRowReference* reference = g_hash_table_lookup(gs_TreeRowReferences, download);
	assert(reference);
	GtkTreePath* path = gtk_tree_row_reference_get_path(reference);

	GtkWidget* scrolledwindow = glade_xml_get_widget(GnaughtyXML, "downloads_scrolled_window");
	GList* children = gtk_container_get_children(GTK_CONTAINER(scrolledwindow));
	GtkListStore* store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(children->data)));
	assert(GTK_IS_LIST_STORE(store));

	GtkTreeIter treeiter;
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &treeiter, path);

	gtk_list_store_remove(store, &treeiter);
	gtk_tree_path_free (path);
}

/**
 * @return true when the movies entry is selected
 */
gboolean gnaughty_gui_is_movies_selected()
{
	if(gtk_option_menu_get_history(GTK_OPTION_MENU(glade_xml_get_widget(GnaughtyXML, "types_optionmenu"))) == MOVIEENTRY) {
		return TRUE;
	} else {
		return FALSE;
	}
}

// vim:noexpandtab:ts=4 sw=4:
